#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé le Tue Apr  4 20:36:35 2023

@auteur: david ALBERTO
(www.astrolabe-science.fr)
"""
import matplotlib.pyplot as plt
import numpy as np
import matplotlib as mpl#Normalisation des valeurs

plt.rcParams["font.family"] = "Roboto" # ou autre police installée
plt.rcParams["font.size"] = 8

# rayons des arcs des ordres
R = range(5,8)

# valeurs du pas du réseau :
a = [2.00, 1.80, 1.60, 1.40]

# longueurs d'onde (nm):
LO = [400, 450, 550, 650]

couleur_fond = 'black'
NomFichier = "diffractionReseauMulti"

# fonctions : -------------------------------------

def angle(p,LO):
    """
    à partir de l'ordre p et de la longueur d'onde LO,
    renvoie l'angle de diffraction, en radians
    """
    A = p * LO*1e-9 / (pas*1e-6)
    if abs(A) <= 1:
        return np.arcsin(A)
    else:
        return np.nan

def ordre(p):
    """
    trace un ordre complet.
    """
    r = R[abs(p)-1]
    for L in LO:
        theta = angle(p,L)
        if np.isnan(theta) == False:
            axe.vlines(theta, 0, r, color=cmap(norm(L)))

def signe(p): # retourne -1 si p < 0 ou 1 si p > 0
    return abs(p)/p

def arc(p):
    """
    Trace un arc pour délimiter l'ordre p.
    """
    r = R[abs(p)-1]
    Lmin=min(LO)
    Lmax=max(LO)
    angle_min = angle(p, Lmin)
    angle_max = angle(p, Lmax)
    if np.isnan(angle_max): # si le dernier rayon est hors champ
        angle_max=np.pi/2 * signe(p)
    angle_moy = (angle_min + angle_max)/2
    angles = np.arange(angle_min,angle_max,0.02 * signe(p))
    for i in angles:
        axe.scatter(i,r+0.1,s=3,c='gray')
    axe.text(angle_moy,r+0.3,f'ordre {p}',c='gray',ha='center',va='center',rotation = -np.degrees(angle_moy))


def format_axe(axe):
    axe.set_facecolor('black')
    axe.set_xlim(np.pi/2, -np.pi/2) # réduire à un demi-cercle
    axe.set_theta_zero_location('N')  # Set zero to North
    axe.set_xticks([])
    # axe.set_yticks(R)
    axe.set_yticks([])
    axe.set_ylim(0,R[-1]+1)
    axe.set_theta_direction(-1)  # theta increasing clockwise
    # ordre 0 :
    axe.vlines(0,0,R[-1],color='white')
    axe.text(0,R[-1]+0.2,'ordre 0',color="white",ha='center')
    axe.text(0.5,0.24,f'pas du réseau : {pas:.2f} µm',c='firebrick',
             va='top', ha='center', fontweight='bold',
             transform=axe.transAxes)

# ------------------- propriétés du graphique

fig, axes = plt.subplots(nrows=2,ncols=2,subplot_kw=dict(projection='polar')
                          # ,tight_layout=True
                         )
fig.set_size_inches(29.7/2.54, 21/2.54) # format a4
# fig.set_size_inches(29.7/2.54, 18/2.54) # format a4

axes = axes.flatten()

plt.subplots_adjust(hspace=-0.70,
                    wspace=0.08,
                    top=1.2,
                    bottom=-0.1
                    )
# ------------------------------------
#Réglage du colormap :
cmap=plt.get_cmap('turbo')# colormap reproduisant les couleurs du visible
norm = mpl.colors.Normalize(vmin=400,vmax=700)#normalisation des couleurs

# ------------------------------------

for N in range(len(axes)):
    axe = axes[N]
    pas = a[N]
    format_axe(axe)
    # tracé des rayons :
    for p in [-3,-2,-1,1,2,3]:
        ordre(p)
        arc(p)




fig.savefig(NomFichier + '.png',dpi=200)
fig.savefig(NomFichier + '.pdf')
